@@ -1,5 +1,7 @@
-from tacotoolbox import datamodel
+from tacotoolbox import datamodel, validator
 from tacotoolbox.create import create
+from tacotoolbox.tacocat import create_tacocat
+from tacotoolbox.tacollection import create_tacollection
 
 
 def _get_version() -> str:
@@ -15,8 +17,4 @@ def _get_version() -> str:
 __version__ = _get_version()
 
 
-__all__ = [
-    "create",
-    "datamodel",
-    "edit",
-]
+__all__ = ["create", "create_tacocat", "create_tacollection", "datamodel", "validator"]
