@@ -1,4 +1,4 @@
-from importlib.resources import files
+from importlib.resources import as_file, files
 
 import polars as pl
 
@@ -9,12 +9,16 @@ def _chunks(seq, size: int):
         yield seq[i : i + size]
 
 
-def load_admin_layer(level: int | str, admin_layers_pkg: str = "tacotoolbox") -> pl.DataFrame:
+def load_admin_layer(
+    level: int | str, admin_layers_pkg: str = "tacotoolbox"
+) -> pl.DataFrame:
     """Load admin lookup table (code->name) for given level (0=countries, 1=states, 2=districts)."""
     lvl = str(level)
     base = files(admin_layers_pkg).joinpath("tortilla/data/admin/")
-    path = base / f"admin{lvl}.parquet"
-    return pl.read_parquet(path)
+    traversable = base / f"admin{lvl}.parquet"
+
+    with as_file(traversable) as path:
+        return pl.read_parquet(path)
 
 
 def morton_key(lon: float, lat: float, bits: int = 24) -> int:
@@ -29,10 +33,10 @@ def morton_key(lon: float, lat: float, bits: int = 24) -> int:
         """Interleave bits by inserting zeros between them."""
         v &= 0x00000000FFFFFFFF
         v = (v | (v << 16)) & 0x0000FFFF0000FFFF
-        v = (v | (v << 8))  & 0x00FF00FF00FF00FF
-        v = (v | (v << 4))  & 0x0F0F0F0F0F0F0F0F
-        v = (v | (v << 2))  & 0x3333333333333333
-        v = (v | (v << 1))  & 0x5555555555555555
+        v = (v | (v << 8)) & 0x00FF00FF00FF00FF
+        v = (v | (v << 4)) & 0x0F0F0F0F0F0F0F0F
+        v = (v | (v << 2)) & 0x3333333333333333
+        v = (v | (v << 1)) & 0x5555555555555555
         return v
 
-    return (_part1by1(xi) << 1) | _part1by1(yi)
\ No newline at end of file
+    return (_part1by1(xi) << 1) | _part1by1(yi)
