@@ -3,11 +3,4 @@ import pytortilla as tortilla
 from tacotoolbox.v1 import datamodel, edit
 from tacotoolbox.v1.create import create, taco2tortilla, tortilla2taco
 
-__all__ = [
-    "datamodel",
-    "create",
-    "tortilla2taco",
-    "taco2tortilla",
-    "tortilla",
-    "edit"
-]
+__all__ = ["datamodel", "create", "tortilla2taco", "taco2tortilla", "tortilla", "edit"]
